--body:getContacts() returns contacts even if bodies are not touching?

lg = love.graphics
rnd=math.random
sin=math.sin
cos=math.cos
local res_w = 1280
local res_h = 720

objects = {}

function love.load()
	love.window.setMode (res_w, res_h, {fullscreen=false})
	world = love.physics.newWorld(0,0, res_w, res_h) 
	world:setGravity(0, 0)
end

function throwBall (x,y,a,speed, params)
	local newBall = {}
	params = params or {}
	newBall.body = love.physics.newBody(world, x,y, "dynamic")
	newBall.shape = love.physics.newCircleShape( 15)
	newBall.fixture = love.physics.newFixture(newBall.body, newBall.shape, 1)
	newBall.fixture:setRestitution(0.9) --let the ball bounce
	newBall.body:setLinearVelocity (sin(a)*speed+rnd(-10,10),cos(a)*speed+rnd(-10,10))	
	newBall.body:setUserData ( {color = {math.random(1,10)/10,math.random(1,10)/10,math.random(1,10)/10,1}} )
	newBall.body:setLinearDamping( 0.4 )
	newBall.color = params.color or {1,0,0,1}
	--table.insert (objects, newBall.body, newBall)
	objects[newBall.body] = newBall
end

function love.update(dt)
	world:update(dt)
end

function love.draw()
	lg.print ("press left/right arrowkeys to spawn balls\n"..
	"red diamonds show contacts as returned by world:getContactList()\n"..
	"number on ball shows #body:getContacts()",10,10)
	for _, body in pairs(world:getBodies()) do
		local ud = body:getUserData()
		lg.setColor (ud.color)
		for _, fixture in pairs(body:getFixtures()) do
			local shape = fixture:getShape()
				if shape:typeOf("CircleShape") then
				local cx, cy = body:getWorldPoints(shape:getPoint())
				love.graphics.circle("fill", cx, cy, shape:getRadius())
			elseif shape:typeOf("PolygonShape") then
				love.graphics.polygon("line", body:getWorldPoints(shape:getPoints()))
			else
				love.graphics.line(body:getWorldPoints(shape:getPoints()))
			end
		end
		--print number of contacts on top of each ball
		local contacts = body:getContacts()
		local x,y=body:getPosition()
		lg.setColor (0,0,0,0.5)
		lg.rectangle ("fill",x,y,10,10)
		lg.setColor (1,1,1,1)
		lg.print (#contacts, body:getPosition() )

	end
	
	--draw contacts
	lg.setColor(255,0,0,255)
	local contacts = world:getContactList()
	for i=1,#contacts do
		local x1,y1,x2,y2 = contacts[i]:getPositions()
		if (x1) then
			lg.circle ("fill",x1,y1,6,4)
		end
		if (x2) then
			lg.circle ("fill",x2,y2,6,4)
		end
	end
end


function love.keypressed (key)
	if key == "escape" then love.event.push("quit") end
	if key == "right" then throwBall (800,res_h/2, math.pi*1.5, 200,{color={1,0,0,0}}) end
	if key == "left" then throwBall (50,res_h/2, math.pi/2, 200, {color={0,1,0,0}}) end
end
